import { z as assign, A as identity, c as create_ssr_component, v as validate_component, b as createEventDispatcher, s as subscribe, d as add_attribute, e as escape, f as add_styles, B as compute_rest_props, C as spread, D as escape_attribute_value, E as escape_object } from './ssr-B2uXEC_g.js';
import './dispatch-DGVRt-1i.js';
import { aA as now, aB as loop, aC as is_date, m as mt$1, b as zA, _ as _e$1, f as bt$1, g as Xt, k as kt, i as Ke$1, n as j, y as pe$1, j as Le$1, C, U as Ut, x as qt$1, A } from './2-C6BxtUu1.js';
import { w as writable } from './index-Bk78Fi56.js';
import { g as ge$1 } from './ModifyUpload-DvDKQOcI.js';
import './Component-CWoGfDVS.js';
import 'tty';
import 'path';
import 'url';
import 'fs';

/** @returns {(t: any) => any} */
function get_interpolator(a, b) {
	if (a === b || a !== a) return () => a;
	const type = typeof a;
	if (type !== typeof b || Array.isArray(a) !== Array.isArray(b)) {
		throw new Error('Cannot interpolate values of different type');
	}
	if (Array.isArray(a)) {
		const arr = b.map((bi, i) => {
			return get_interpolator(a[i], bi);
		});
		return (t) => arr.map((fn) => fn(t));
	}
	if (type === 'object') {
		if (!a || !b) throw new Error('Object cannot be null');
		if (is_date(a) && is_date(b)) {
			a = a.getTime();
			b = b.getTime();
			const delta = b - a;
			return (t) => new Date(a + t * delta);
		}
		const keys = Object.keys(b);
		const interpolators = {};
		keys.forEach((key) => {
			interpolators[key] = get_interpolator(a[key], b[key]);
		});
		return (t) => {
			const result = {};
			keys.forEach((key) => {
				result[key] = interpolators[key](t);
			});
			return result;
		};
	}
	if (type === 'number') {
		const delta = b - a;
		return (t) => a + t * delta;
	}
	throw new Error(`Cannot interpolate ${type} values`);
}

/**
 * A tweened store in Svelte is a special type of store that provides smooth transitions between state values over time.
 *
 * https://svelte.dev/docs/svelte-motion#tweened
 * @template T
 * @param {T} [value]
 * @param {import('./private.js').TweenedOptions<T>} [defaults]
 * @returns {import('./public.js').Tweened<T>}
 */
function tweened(value, defaults = {}) {
	const store = writable(value);
	/** @type {import('../internal/private.js').Task} */
	let task;
	let target_value = value;
	/**
	 * @param {T} new_value
	 * @param {import('./private.js').TweenedOptions<T>} [opts]
	 */
	function set(new_value, opts) {
		if (value == null) {
			store.set((value = new_value));
			return Promise.resolve();
		}
		target_value = new_value;
		let previous_task = task;
		let started = false;
		let {
			delay = 0,
			duration = 400,
			easing = identity,
			interpolate = get_interpolator
		} = assign(assign({}, defaults), opts);
		if (duration === 0) {
			if (previous_task) {
				previous_task.abort();
				previous_task = null;
			}
			store.set((value = target_value));
			return Promise.resolve();
		}
		const start = now() + delay;
		let fn;
		task = loop((now) => {
			if (now < start) return true;
			if (!started) {
				fn = interpolate(value, new_value);
				if (typeof duration === 'function') duration = duration(value, new_value);
				started = true;
			}
			if (previous_task) {
				previous_task.abort();
				previous_task = null;
			}
			const elapsed = now - start;
			if (elapsed > /** @type {number} */ (duration)) {
				store.set((value = new_value));
				return false;
			}
			// @ts-ignore
			store.set((value = fn(easing(elapsed / duration))));
			return true;
		});
		return task.promise;
	}
	return {
		set,
		update: (fn, opts) => set(fn(target_value, value), opts),
		subscribe: store.subscribe
	};
}

var nt="http://www.w3.org/1999/xhtml";const ct={svg:"http://www.w3.org/2000/svg",xhtml:nt,xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"};function mt(e){var t=e+="",n=t.indexOf(":");return n>=0&&(t=e.slice(0,n))!=="xmlns"&&(e=e.slice(n+1)),ct.hasOwnProperty(t)?{space:ct[t],local:e}:e}function Nt(e){return function(){var t=this.ownerDocument,n=this.namespaceURI;return n===nt&&t.documentElement.namespaceURI===nt?t.createElement(e):t.createElementNS(n,e)}}function Pt(e){return function(){return this.ownerDocument.createElementNS(e.space,e.local)}}function vt(e){var t=mt(e);return (t.local?Pt:Nt)(t)}function Vt(){}function Ct(e){return e==null?Vt:function(){return this.querySelector(e)}}function Ft(e){typeof e!="function"&&(e=Ct(e));for(var t=this._groups,n=t.length,o=new Array(n),i=0;i<n;++i)for(var l=t[i],a=l.length,r=o[i]=new Array(a),s,d,u=0;u<a;++u)(s=l[u])&&(d=e.call(s,s.__data__,u,l))&&("__data__"in s&&(d.__data__=s.__data__),r[u]=d);return new X(o,this._parents)}function Zt(e){return e==null?[]:Array.isArray(e)?e:Array.from(e)}function Ht(){return []}function qt(e){return e==null?Ht:function(){return this.querySelectorAll(e)}}function Jt(e){return function(){return Zt(e.apply(this,arguments))}}function jt(e){typeof e=="function"?e=Jt(e):e=qt(e);for(var t=this._groups,n=t.length,o=[],i=[],l=0;l<n;++l)for(var a=t[l],r=a.length,s,d=0;d<r;++d)(s=a[d])&&(o.push(e.call(s,s.__data__,d,a)),i.push(s));return new X(o,i)}function $t(e){return function(){return this.matches(e)}}function gt(e){return function(t){return t.matches(e)}}var te=Array.prototype.find;function ee(e){return function(){return te.call(this.children,e)}}function ne(){return this.firstElementChild}function ie(e){return this.select(e==null?ne:ee(typeof e=="function"?e:gt(e)))}var oe=Array.prototype.filter;function le(){return Array.from(this.children)}function ae(e){return function(){return oe.call(this.children,e)}}function re(e){return this.selectAll(e==null?le:ae(typeof e=="function"?e:gt(e)))}function se(e){typeof e!="function"&&(e=$t(e));for(var t=this._groups,n=t.length,o=new Array(n),i=0;i<n;++i)for(var l=t[i],a=l.length,r=o[i]=[],s,d=0;d<a;++d)(s=l[d])&&e.call(s,s.__data__,d,l)&&r.push(s);return new X(o,this._parents)}function pt(e){return new Array(e.length)}function ce(){return new X(this._enter||this._groups.map(pt),this._parents)}function q(e,t){this.ownerDocument=e.ownerDocument,this.namespaceURI=e.namespaceURI,this._next=null,this._parent=e,this.__data__=t;}q.prototype={constructor:q,appendChild:function(e){return this._parent.insertBefore(e,this._next)},insertBefore:function(e,t){return this._parent.insertBefore(e,t)},querySelector:function(e){return this._parent.querySelector(e)},querySelectorAll:function(e){return this._parent.querySelectorAll(e)}};function de(e){return function(){return e}}function ue(e,t,n,o,i,l){for(var a=0,r,s=t.length,d=l.length;a<d;++a)(r=t[a])?(r.__data__=l[a],o[a]=r):n[a]=new q(e,l[a]);for(;a<s;++a)(r=t[a])&&(i[a]=r);}function Ae(e,t,n,o,i,l,a){var r,s,d=new Map,u=t.length,f=l.length,A=new Array(u),h;for(r=0;r<u;++r)(s=t[r])&&(A[r]=h=a.call(s,s.__data__,r,t)+"",d.has(h)?i[r]=s:d.set(h,s));for(r=0;r<f;++r)h=a.call(e,l[r],r,l)+"",(s=d.get(h))?(o[r]=s,s.__data__=l[r],d.delete(h)):n[r]=new q(e,l[r]);for(r=0;r<u;++r)(s=t[r])&&d.get(A[r])===s&&(i[r]=s);}function he(e){return e.__data__}function fe(e,t){if(!arguments.length)return Array.from(this,he);var n=t?Ae:ue,o=this._parents,i=this._groups;typeof e!="function"&&(e=de(e));for(var l=i.length,a=new Array(l),r=new Array(l),s=new Array(l),d=0;d<l;++d){var u=o[d],f=i[d],A=f.length,h=_e(e.call(u,u&&u.__data__,d,o)),_=h.length,b=r[d]=new Array(_),w=a[d]=new Array(_),y=s[d]=new Array(A);n(u,f,b,w,y,h,t);for(var x=0,z=0,c,m;x<_;++x)if(c=b[x]){for(x>=z&&(z=x+1);!(m=w[z])&&++z<_;);c._next=m||null;}}return a=new X(a,o),a._enter=r,a._exit=s,a}function _e(e){return typeof e=="object"&&"length"in e?e:Array.from(e)}function me(){return new X(this._exit||this._groups.map(pt),this._parents)}function ve(e,t,n){var o=this.enter(),i=this,l=this.exit();return typeof e=="function"?(o=e(o),o&&(o=o.selection())):o=o.append(e+""),t!=null&&(i=t(i),i&&(i=i.selection())),n==null?l.remove():n(l),o&&i?o.merge(i).order():i}function Ce(e){for(var t=e.selection?e.selection():e,n=this._groups,o=t._groups,i=n.length,l=o.length,a=Math.min(i,l),r=new Array(i),s=0;s<a;++s)for(var d=n[s],u=o[s],f=d.length,A=r[s]=new Array(f),h,_=0;_<f;++_)(h=d[_]||u[_])&&(A[_]=h);for(;s<i;++s)r[s]=n[s];return new X(r,this._parents)}function ge(){for(var e=this._groups,t=-1,n=e.length;++t<n;)for(var o=e[t],i=o.length-1,l=o[i],a;--i>=0;)(a=o[i])&&(l&&a.compareDocumentPosition(l)^4&&l.parentNode.insertBefore(a,l),l=a);return this}function pe(e){e||(e=we);function t(f,A){return f&&A?e(f.__data__,A.__data__):!f-!A}for(var n=this._groups,o=n.length,i=new Array(o),l=0;l<o;++l){for(var a=n[l],r=a.length,s=i[l]=new Array(r),d,u=0;u<r;++u)(d=a[u])&&(s[u]=d);s.sort(t);}return new X(i,this._parents).order()}function we(e,t){return e<t?-1:e>t?1:e>=t?0:NaN}function xe(){var e=arguments[0];return arguments[0]=this,e.apply(null,arguments),this}function ye(){return Array.from(this)}function be(){for(var e=this._groups,t=0,n=e.length;t<n;++t)for(var o=e[t],i=0,l=o.length;i<l;++i){var a=o[i];if(a)return a}return null}function Ee(){let e=0;for(const t of this)++e;return e}function Ie(){return !this.node()}function Be(e){for(var t=this._groups,n=0,o=t.length;n<o;++n)for(var i=t[n],l=0,a=i.length,r;l<a;++l)(r=i[l])&&e.call(r,r.__data__,l,i);return this}function ze(e){return function(){this.removeAttribute(e);}}function Me(e){return function(){this.removeAttributeNS(e.space,e.local);}}function Se(e,t){return function(){this.setAttribute(e,t);}}function ke(e,t){return function(){this.setAttributeNS(e.space,e.local,t);}}function Oe(e,t){return function(){var n=t.apply(this,arguments);n==null?this.removeAttribute(e):this.setAttribute(e,n);}}function Xe(e,t){return function(){var n=t.apply(this,arguments);n==null?this.removeAttributeNS(e.space,e.local):this.setAttributeNS(e.space,e.local,n);}}function Ke(e,t){var n=mt(e);if(arguments.length<2){var o=this.node();return n.local?o.getAttributeNS(n.space,n.local):o.getAttribute(n)}return this.each((t==null?n.local?Me:ze:typeof t=="function"?n.local?Xe:Oe:n.local?ke:Se)(n,t))}function wt(e){return e.ownerDocument&&e.ownerDocument.defaultView||e.document&&e||e.defaultView}function Re(e){return function(){this.style.removeProperty(e);}}function Ye(e,t,n){return function(){this.style.setProperty(e,t,n);}}function Ue(e,t,n){return function(){var o=t.apply(this,arguments);o==null?this.style.removeProperty(e):this.style.setProperty(e,o,n);}}function De(e,t,n){return arguments.length>1?this.each((t==null?Re:typeof t=="function"?Ue:Ye)(e,t,n??"")):Qe(this.node(),e)}function Qe(e,t){return e.style.getPropertyValue(t)||wt(e).getComputedStyle(e,null).getPropertyValue(t)}function Ge(e){return function(){delete this[e];}}function Le(e,t){return function(){this[e]=t;}}function Te(e,t){return function(){var n=t.apply(this,arguments);n==null?delete this[e]:this[e]=n;}}function We(e,t){return arguments.length>1?this.each((t==null?Ge:typeof t=="function"?Te:Le)(e,t)):this.node()[e]}function xt(e){return e.trim().split(/^|\s+/)}function lt(e){return e.classList||new yt(e)}function yt(e){this._node=e,this._names=xt(e.getAttribute("class")||"");}yt.prototype={add:function(e){var t=this._names.indexOf(e);t<0&&(this._names.push(e),this._node.setAttribute("class",this._names.join(" ")));},remove:function(e){var t=this._names.indexOf(e);t>=0&&(this._names.splice(t,1),this._node.setAttribute("class",this._names.join(" ")));},contains:function(e){return this._names.indexOf(e)>=0}};function bt(e,t){for(var n=lt(e),o=-1,i=t.length;++o<i;)n.add(t[o]);}function Et(e,t){for(var n=lt(e),o=-1,i=t.length;++o<i;)n.remove(t[o]);}function Ne(e){return function(){bt(this,e);}}function Pe(e){return function(){Et(this,e);}}function Ve(e,t){return function(){(t.apply(this,arguments)?bt:Et)(this,e);}}function Fe(e,t){var n=xt(e+"");if(arguments.length<2){for(var o=lt(this.node()),i=-1,l=n.length;++i<l;)if(!o.contains(n[i]))return !1;return !0}return this.each((typeof t=="function"?Ve:t?Ne:Pe)(n,t))}function Ze(){this.textContent="";}function He(e){return function(){this.textContent=e;}}function qe(e){return function(){var t=e.apply(this,arguments);this.textContent=t??"";}}function Je(e){return arguments.length?this.each(e==null?Ze:(typeof e=="function"?qe:He)(e)):this.node().textContent}function je(){this.innerHTML="";}function $e(e){return function(){this.innerHTML=e;}}function tn(e){return function(){var t=e.apply(this,arguments);this.innerHTML=t??"";}}function en(e){return arguments.length?this.each(e==null?je:(typeof e=="function"?tn:$e)(e)):this.node().innerHTML}function nn(){this.nextSibling&&this.parentNode.appendChild(this);}function on(){return this.each(nn)}function ln(){this.previousSibling&&this.parentNode.insertBefore(this,this.parentNode.firstChild);}function an(){return this.each(ln)}function rn(e){var t=typeof e=="function"?e:vt(e);return this.select(function(){return this.appendChild(t.apply(this,arguments))})}function sn(){return null}function cn(e,t){var n=typeof e=="function"?e:vt(e),o=t==null?sn:typeof t=="function"?t:Ct(t);return this.select(function(){return this.insertBefore(n.apply(this,arguments),o.apply(this,arguments)||null)})}function dn(){var e=this.parentNode;e&&e.removeChild(this);}function un(){return this.each(dn)}function An(){var e=this.cloneNode(!1),t=this.parentNode;return t?t.insertBefore(e,this.nextSibling):e}function hn(){var e=this.cloneNode(!0),t=this.parentNode;return t?t.insertBefore(e,this.nextSibling):e}function fn(e){return this.select(e?hn:An)}function _n(e){return arguments.length?this.property("__data__",e):this.node().__data__}function mn(e){return function(t){e.call(this,t,this.__data__);}}function vn(e){return e.trim().split(/^|\s+/).map(function(t){var n="",o=t.indexOf(".");return o>=0&&(n=t.slice(o+1),t=t.slice(0,o)),{type:t,name:n}})}function Cn(e){return function(){var t=this.__on;if(t){for(var n=0,o=-1,i=t.length,l;n<i;++n)l=t[n],(!e.type||l.type===e.type)&&l.name===e.name?this.removeEventListener(l.type,l.listener,l.options):t[++o]=l;++o?t.length=o:delete this.__on;}}}function gn(e,t,n){return function(){var o=this.__on,i,l=mn(t);if(o){for(var a=0,r=o.length;a<r;++a)if((i=o[a]).type===e.type&&i.name===e.name){this.removeEventListener(i.type,i.listener,i.options),this.addEventListener(i.type,i.listener=l,i.options=n),i.value=t;return}}this.addEventListener(e.type,l,n),i={type:e.type,name:e.name,value:t,listener:l,options:n},o?o.push(i):this.__on=[i];}}function pn(e,t,n){var o=vn(e+""),i,l=o.length,a;if(arguments.length<2){var r=this.node().__on;if(r){for(var s=0,d=r.length,u;s<d;++s)for(i=0,u=r[s];i<l;++i)if((a=o[i]).type===u.type&&a.name===u.name)return u.value}return}for(r=t?gn:Cn,i=0;i<l;++i)this.each(r(o[i],t,n));return this}function It(e,t,n){var o=wt(e),i=o.CustomEvent;typeof i=="function"?i=new i(t,n):(i=o.document.createEvent("Event"),n?(i.initEvent(t,n.bubbles,n.cancelable),i.detail=n.detail):i.initEvent(t,!1,!1)),e.dispatchEvent(i);}function wn(e,t){return function(){return It(this,e,t)}}function xn(e,t){return function(){return It(this,e,t.apply(this,arguments))}}function yn(e,t){return this.each((typeof t=="function"?xn:wn)(e,t))}function*bn(){for(var e=this._groups,t=0,n=e.length;t<n;++t)for(var o=e[t],i=0,l=o.length,a;i<l;++i)(a=o[i])&&(yield a);}function X(e,t){this._groups=e,this._parents=t;}function In(){return this}X.prototype={constructor:X,select:Ft,selectAll:jt,selectChild:ie,selectChildren:re,filter:se,data:fe,enter:ce,exit:me,join:ve,merge:Ce,selection:In,order:ge,sort:pe,call:xe,nodes:ye,node:be,size:Ee,empty:Ie,each:Be,attr:Ke,style:De,property:We,classed:Fe,text:Je,html:en,raise:on,lower:an,append:rn,insert:cn,remove:un,clone:fn,datum:_n,on:pn,dispatch:yn,[Symbol.iterator]:bn};const Yn={code:".wrap.svelte-fpmna9.svelte-fpmna9{position:relative;width:100%;height:100%;z-index:var(--layer-1);overflow:hidden}.icon-wrap.svelte-fpmna9.svelte-fpmna9{display:block;position:absolute;top:50%;transform:translate(-20.5px, -50%);left:10px;width:40px;transition:0.2s;color:var(--body-text-color);height:30px;border-radius:5px;background-color:var(--color-accent);display:flex;align-items:center;justify-content:center;z-index:var(--layer-3);box-shadow:0px 0px 5px 2px rgba(0, 0, 0, 0.3);font-size:12px}.icon.left.svelte-fpmna9.svelte-fpmna9{transform:rotate(135deg);text-shadow:-1px -1px 1px rgba(0, 0, 0, 0.1)}.icon.right.svelte-fpmna9.svelte-fpmna9{transform:rotate(-45deg);text-shadow:-1px -1px 1px rgba(0, 0, 0, 0.1)}.icon.center.svelte-fpmna9.svelte-fpmna9{display:block;width:1px;height:100%;background-color:var(--color);opacity:0.1}.icon-wrap.active.svelte-fpmna9.svelte-fpmna9{opacity:0}.icon-wrap.disabled.svelte-fpmna9.svelte-fpmna9{opacity:0}.outer.svelte-fpmna9.svelte-fpmna9{width:20px;height:100%;position:absolute;cursor:grab;position:absolute;top:0;left:-10px;pointer-events:auto;z-index:var(--layer-2)}.grab.svelte-fpmna9.svelte-fpmna9{cursor:grabbing}.inner.svelte-fpmna9.svelte-fpmna9{width:1px;height:100%;background:var(--color);position:absolute;left:calc((100% - 2px) / 2)}.disabled.svelte-fpmna9.svelte-fpmna9{cursor:auto}.disabled.svelte-fpmna9 .inner.svelte-fpmna9{box-shadow:none}.content.svelte-fpmna9.svelte-fpmna9{width:100%;height:100%;display:flex;justify-content:center;align-items:center}",map:'{"version":3,"file":"Slider.svelte","sources":["Slider.svelte"],"sourcesContent":["<script lang=\\"ts\\">import { onMount } from \\"svelte\\";\\nimport { drag } from \\"d3-drag\\";\\nimport { select } from \\"d3-selection\\";\\nfunction clamp(value, min, max) {\\n    return Math.min(Math.max(value, min), max);\\n}\\nexport let position = 0.5;\\nexport let disabled = false;\\nexport let slider_color = \\"var(--border-color-primary)\\";\\nexport let image_size = { top: 0, left: 0, width: 0, height: 0 };\\nexport let el = void 0;\\nexport let parent_el = void 0;\\nlet inner;\\nlet px = 0;\\nlet active = false;\\nlet container_width = 0;\\nfunction set_position(width) {\\n    container_width = parent_el?.getBoundingClientRect().width || 0;\\n    if (width === 0) {\\n        image_size.width = el?.getBoundingClientRect().width || 0;\\n    }\\n    px = clamp(image_size.width * position + image_size.left, 0, container_width);\\n}\\nfunction round(n, points) {\\n    const mod = Math.pow(10, points);\\n    return Math.round((n + Number.EPSILON) * mod) / mod;\\n}\\nfunction update_position(x) {\\n    px = clamp(x, 0, container_width);\\n    position = round((x - image_size.left) / image_size.width, 5);\\n}\\nfunction drag_start(event) {\\n    if (disabled)\\n        return;\\n    active = true;\\n    update_position(event.x);\\n}\\nfunction drag_move(event) {\\n    if (disabled)\\n        return;\\n    update_position(event.x);\\n}\\nfunction drag_end() {\\n    if (disabled)\\n        return;\\n    active = false;\\n}\\nfunction update_position_from_pc(pc) {\\n    px = clamp(image_size.width * pc + image_size.left, 0, container_width);\\n}\\n$: set_position(image_size.width);\\n$: update_position_from_pc(position);\\nonMount(() => {\\n    set_position(image_size.width);\\n    const drag_handler = drag().on(\\"start\\", drag_start).on(\\"drag\\", drag_move).on(\\"end\\", drag_end);\\n    select(inner).call(drag_handler);\\n});\\n<\/script>\\n\\n<svelte:window on:resize={() => set_position(image_size.width)} />\\n\\n<div class=\\"wrap\\" role=\\"none\\" bind:this={parent_el}>\\n\\t<div class=\\"content\\" bind:this={el}>\\n\\t\\t<slot />\\n\\t</div>\\n\\t<div\\n\\t\\tclass=\\"outer\\"\\n\\t\\tclass:disabled\\n\\t\\tbind:this={inner}\\n\\t\\trole=\\"none\\"\\n\\t\\tstyle=\\"transform: translateX({px}px)\\"\\n\\t\\tclass:grab={active}\\n\\t>\\n\\t\\t<span class=\\"icon-wrap\\" class:active class:disabled\\n\\t\\t\\t><span class=\\"icon left\\">◢</span><span\\n\\t\\t\\t\\tclass=\\"icon center\\"\\n\\t\\t\\t\\tstyle:--color={slider_color}\\n\\t\\t\\t></span><span class=\\"icon right\\">◢</span></span\\n\\t\\t>\\n\\t\\t<div class=\\"inner\\" style:--color={slider_color}></div>\\n\\t</div>\\n</div>\\n\\n<style>\\n\\t.wrap {\\n\\t\\tposition: relative;\\n\\t\\twidth: 100%;\\n\\t\\theight: 100%;\\n\\t\\tz-index: var(--layer-1);\\n\\t\\toverflow: hidden;\\n\\t}\\n\\n\\t.icon-wrap {\\n\\t\\tdisplay: block;\\n\\t\\tposition: absolute;\\n\\t\\ttop: 50%;\\n\\t\\ttransform: translate(-20.5px, -50%);\\n\\t\\tleft: 10px;\\n\\t\\twidth: 40px;\\n\\t\\ttransition: 0.2s;\\n\\t\\tcolor: var(--body-text-color);\\n\\t\\theight: 30px;\\n\\t\\tborder-radius: 5px;\\n\\t\\tbackground-color: var(--color-accent);\\n\\t\\tdisplay: flex;\\n\\t\\talign-items: center;\\n\\t\\tjustify-content: center;\\n\\t\\tz-index: var(--layer-3);\\n\\t\\tbox-shadow: 0px 0px 5px 2px rgba(0, 0, 0, 0.3);\\n\\t\\tfont-size: 12px;\\n\\t}\\n\\n\\t.icon.left {\\n\\t\\ttransform: rotate(135deg);\\n\\t\\ttext-shadow: -1px -1px 1px rgba(0, 0, 0, 0.1);\\n\\t}\\n\\n\\t.icon.right {\\n\\t\\ttransform: rotate(-45deg);\\n\\t\\ttext-shadow: -1px -1px 1px rgba(0, 0, 0, 0.1);\\n\\t}\\n\\n\\t.icon.center {\\n\\t\\tdisplay: block;\\n\\t\\twidth: 1px;\\n\\t\\theight: 100%;\\n\\t\\tbackground-color: var(--color);\\n\\t\\topacity: 0.1;\\n\\t}\\n\\n\\t.icon-wrap.active {\\n\\t\\topacity: 0;\\n\\t}\\n\\n\\t.icon-wrap.disabled {\\n\\t\\topacity: 0;\\n\\t}\\n\\n\\t.outer {\\n\\t\\twidth: 20px;\\n\\t\\theight: 100%;\\n\\t\\tposition: absolute;\\n\\t\\tcursor: grab;\\n\\t\\tposition: absolute;\\n\\t\\ttop: 0;\\n\\t\\tleft: -10px;\\n\\t\\tpointer-events: auto;\\n\\t\\tz-index: var(--layer-2);\\n\\t}\\n\\t.grab {\\n\\t\\tcursor: grabbing;\\n\\t}\\n\\n\\t.inner {\\n\\t\\twidth: 1px;\\n\\t\\theight: 100%;\\n\\t\\tbackground: var(--color);\\n\\t\\tposition: absolute;\\n\\t\\tleft: calc((100% - 2px) / 2);\\n\\t}\\n\\n\\t.disabled {\\n\\t\\tcursor: auto;\\n\\t}\\n\\n\\t.disabled .inner {\\n\\t\\tbox-shadow: none;\\n\\t}\\n\\n\\t.content {\\n\\t\\twidth: 100%;\\n\\t\\theight: 100%;\\n\\t\\tdisplay: flex;\\n\\t\\tjustify-content: center;\\n\\t\\talign-items: center;\\n\\t}</style>\\n"],"names":[],"mappings":"AAoFC,iCAAM,CACL,QAAQ,CAAE,QAAQ,CAClB,KAAK,CAAE,IAAI,CACX,MAAM,CAAE,IAAI,CACZ,OAAO,CAAE,IAAI,SAAS,CAAC,CACvB,QAAQ,CAAE,MACX,CAEA,sCAAW,CACV,OAAO,CAAE,KAAK,CACd,QAAQ,CAAE,QAAQ,CAClB,GAAG,CAAE,GAAG,CACR,SAAS,CAAE,UAAU,OAAO,CAAC,CAAC,IAAI,CAAC,CACnC,IAAI,CAAE,IAAI,CACV,KAAK,CAAE,IAAI,CACX,UAAU,CAAE,IAAI,CAChB,KAAK,CAAE,IAAI,iBAAiB,CAAC,CAC7B,MAAM,CAAE,IAAI,CACZ,aAAa,CAAE,GAAG,CAClB,gBAAgB,CAAE,IAAI,cAAc,CAAC,CACrC,OAAO,CAAE,IAAI,CACb,WAAW,CAAE,MAAM,CACnB,eAAe,CAAE,MAAM,CACvB,OAAO,CAAE,IAAI,SAAS,CAAC,CACvB,UAAU,CAAE,GAAG,CAAC,GAAG,CAAC,GAAG,CAAC,GAAG,CAAC,KAAK,CAAC,CAAC,CAAC,CAAC,CAAC,CAAC,CAAC,CAAC,CAAC,GAAG,CAAC,CAC9C,SAAS,CAAE,IACZ,CAEA,KAAK,iCAAM,CACV,SAAS,CAAE,OAAO,MAAM,CAAC,CACzB,WAAW,CAAE,IAAI,CAAC,IAAI,CAAC,GAAG,CAAC,KAAK,CAAC,CAAC,CAAC,CAAC,CAAC,CAAC,CAAC,CAAC,CAAC,GAAG,CAC7C,CAEA,KAAK,kCAAO,CACX,SAAS,CAAE,OAAO,MAAM,CAAC,CACzB,WAAW,CAAE,IAAI,CAAC,IAAI,CAAC,GAAG,CAAC,KAAK,CAAC,CAAC,CAAC,CAAC,CAAC,CAAC,CAAC,CAAC,CAAC,GAAG,CAC7C,CAEA,KAAK,mCAAQ,CACZ,OAAO,CAAE,KAAK,CACd,KAAK,CAAE,GAAG,CACV,MAAM,CAAE,IAAI,CACZ,gBAAgB,CAAE,IAAI,OAAO,CAAC,CAC9B,OAAO,CAAE,GACV,CAEA,UAAU,mCAAQ,CACjB,OAAO,CAAE,CACV,CAEA,UAAU,qCAAU,CACnB,OAAO,CAAE,CACV,CAEA,kCAAO,CACN,KAAK,CAAE,IAAI,CACX,MAAM,CAAE,IAAI,CACZ,QAAQ,CAAE,QAAQ,CAClB,MAAM,CAAE,IAAI,CACZ,QAAQ,CAAE,QAAQ,CAClB,GAAG,CAAE,CAAC,CACN,IAAI,CAAE,KAAK,CACX,cAAc,CAAE,IAAI,CACpB,OAAO,CAAE,IAAI,SAAS,CACvB,CACA,iCAAM,CACL,MAAM,CAAE,QACT,CAEA,kCAAO,CACN,KAAK,CAAE,GAAG,CACV,MAAM,CAAE,IAAI,CACZ,UAAU,CAAE,IAAI,OAAO,CAAC,CACxB,QAAQ,CAAE,QAAQ,CAClB,IAAI,CAAE,KAAK,CAAC,IAAI,CAAC,CAAC,CAAC,GAAG,CAAC,CAAC,CAAC,CAAC,CAAC,CAC5B,CAEA,qCAAU,CACT,MAAM,CAAE,IACT,CAEA,uBAAS,CAAC,oBAAO,CAChB,UAAU,CAAE,IACb,CAEA,oCAAS,CACR,KAAK,CAAE,IAAI,CACX,MAAM,CAAE,IAAI,CACZ,OAAO,CAAE,IAAI,CACb,eAAe,CAAE,MAAM,CACvB,WAAW,CAAE,MACd"}'};function et(e,t,n){return Math.min(Math.max(e,t),n)}const Bt=create_ssr_component((e,t,n,o)=>{let {position:i=.5}=t,{disabled:l=!1}=t,{slider_color:a="var(--border-color-primary)"}=t,{image_size:r={top:0,left:0,width:0,height:0}}=t,{el:s=void 0}=t,{parent_el:d=void 0}=t,u,f=0,h=0;function _(c){h=d?.getBoundingClientRect().width||0,c===0&&(r.width=s?.getBoundingClientRect().width||0),f=et(r.width*i+r.left,0,h);}function z(c){f=et(r.width*c+r.left,0,h);}return t.position===void 0&&n.position&&i!==void 0&&n.position(i),t.disabled===void 0&&n.disabled&&l!==void 0&&n.disabled(l),t.slider_color===void 0&&n.slider_color&&a!==void 0&&n.slider_color(a),t.image_size===void 0&&n.image_size&&r!==void 0&&n.image_size(r),t.el===void 0&&n.el&&s!==void 0&&n.el(s),t.parent_el===void 0&&n.parent_el&&d!==void 0&&n.parent_el(d),e.css.add(Yn),_(r.width),z(i),` <div class="wrap svelte-fpmna9" role="none"${add_attribute("this",d,0)}><div class="content svelte-fpmna9"${add_attribute("this",s,0)}>${o.default?o.default({}):""}</div> <div class="${["outer svelte-fpmna9",(l?"disabled":"")+" "+("")].join(" ").trim()}" role="none" style="${"transform: translateX("+escape(f,!0)+"px)"}"${add_attribute("this",u,0)}><span class="${["icon-wrap svelte-fpmna9",("")+" "+(l?"disabled":"")].join(" ").trim()}"><span class="icon left svelte-fpmna9" data-svelte-h="svelte-9lsvah">◢</span><span class="icon center svelte-fpmna9"${add_styles({"--color":a})}></span><span class="icon right svelte-fpmna9" data-svelte-h="svelte-1lu38by">◢</span></span> <div class="inner svelte-fpmna9"${add_styles({"--color":a})}></div></div> </div>`}),Dn={code:".preview.svelte-k63p1v{object-fit:contain;width:100%;transform-origin:top left;margin:auto}.small.svelte-k63p1v{max-height:500px}.upload.svelte-k63p1v{object-fit:contain;max-height:500px}.fixed.svelte-k63p1v{position:absolute;top:0;left:0;right:0;bottom:0}.fullscreen.svelte-k63p1v{width:100%;height:100%}.image-container:fullscreen img.svelte-k63p1v{width:100%;height:100%;max-height:none;max-width:none}.hidden.svelte-k63p1v{opacity:0}",map:'{"version":3,"file":"ImageEl.svelte","sources":["ImageEl.svelte"],"sourcesContent":["<script lang=\\"ts\\">import { createEventDispatcher, onMount, tick } from \\"svelte\\";\\nimport { resolve_wasm_src } from \\"@gradio/wasm/svelte\\";\\nexport let src = void 0;\\nexport let fullscreen = false;\\nlet resolved_src;\\nexport let fixed = false;\\nexport let transform = \\"translate(0px, 0px) scale(1)\\";\\nexport let img_el = null;\\nexport let hidden = false;\\nexport let variant = \\"upload\\";\\nexport let max_height = 500;\\nlet latest_src;\\n$: {\\n    resolved_src = src;\\n    latest_src = src;\\n    const resolving_src = src;\\n    resolve_wasm_src(resolving_src).then((s) => {\\n        if (latest_src === resolving_src) {\\n            resolved_src = s;\\n        }\\n    });\\n}\\nconst dispatch = createEventDispatcher();\\nfunction get_image_size(img) {\\n    if (!img)\\n        return { top: 0, left: 0, width: 0, height: 0 };\\n    const container = img.parentElement?.getBoundingClientRect();\\n    if (!container)\\n        return { top: 0, left: 0, width: 0, height: 0 };\\n    const naturalAspect = img.naturalWidth / img.naturalHeight;\\n    const containerAspect = container.width / container.height;\\n    let displayedWidth, displayedHeight;\\n    if (naturalAspect > containerAspect) {\\n        displayedWidth = container.width;\\n        displayedHeight = container.width / naturalAspect;\\n    }\\n    else {\\n        displayedHeight = container.height;\\n        displayedWidth = container.height * naturalAspect;\\n    }\\n    const offsetX = (container.width - displayedWidth) / 2;\\n    const offsetY = (container.height - displayedHeight) / 2;\\n    return {\\n        top: offsetY,\\n        left: offsetX,\\n        width: displayedWidth,\\n        height: displayedHeight\\n    };\\n}\\nonMount(() => {\\n    const resizer = new ResizeObserver(async (entries) => {\\n        for (const entry of entries) {\\n            await tick();\\n            dispatch(\\"load\\", get_image_size(img_el));\\n        }\\n    });\\n    resizer.observe(img_el);\\n    return () => {\\n        resizer.disconnect();\\n    };\\n});\\n<\/script>\\n\\n<!-- svelte-ignore a11y-missing-attribute -->\\n<img\\n\\tsrc={resolved_src}\\n\\t{...$$restProps}\\n\\tclass:fixed\\n\\tstyle:transform\\n\\tbind:this={img_el}\\n\\tclass:hidden\\n\\tclass:preview={variant === \\"preview\\"}\\n\\tclass:slider={variant === \\"upload\\"}\\n\\tstyle:max-height={max_height && !fullscreen ? `${max_height}px` : null}\\n\\tclass:fullscreen\\n\\tclass:small={!fullscreen}\\n\\ton:load={() => dispatch(\\"load\\", get_image_size(img_el))}\\n/>\\n\\n<style>\\n\\t.preview {\\n\\t\\tobject-fit: contain;\\n\\t\\twidth: 100%;\\n\\t\\ttransform-origin: top left;\\n\\t\\tmargin: auto;\\n\\t}\\n\\n\\t.small {\\n\\t\\tmax-height: 500px;\\n\\t}\\n\\n\\t.upload {\\n\\t\\tobject-fit: contain;\\n\\t\\tmax-height: 500px;\\n\\t}\\n\\n\\t.fixed {\\n\\t\\tposition: absolute;\\n\\t\\ttop: 0;\\n\\t\\tleft: 0;\\n\\t\\tright: 0;\\n\\t\\tbottom: 0;\\n\\t}\\n\\n\\t.fullscreen {\\n\\t\\twidth: 100%;\\n\\t\\theight: 100%;\\n\\t}\\n\\n\\t:global(.image-container:fullscreen) img {\\n\\t\\twidth: 100%;\\n\\t\\theight: 100%;\\n\\t\\tmax-height: none;\\n\\t\\tmax-width: none;\\n\\t}\\n\\n\\t.hidden {\\n\\t\\topacity: 0;\\n\\t}</style>\\n"],"names":[],"mappings":"AAgFC,sBAAS,CACR,UAAU,CAAE,OAAO,CACnB,KAAK,CAAE,IAAI,CACX,gBAAgB,CAAE,GAAG,CAAC,IAAI,CAC1B,MAAM,CAAE,IACT,CAEA,oBAAO,CACN,UAAU,CAAE,KACb,CAEA,qBAAQ,CACP,UAAU,CAAE,OAAO,CACnB,UAAU,CAAE,KACb,CAEA,oBAAO,CACN,QAAQ,CAAE,QAAQ,CAClB,GAAG,CAAE,CAAC,CACN,IAAI,CAAE,CAAC,CACP,KAAK,CAAE,CAAC,CACR,MAAM,CAAE,CACT,CAEA,yBAAY,CACX,KAAK,CAAE,IAAI,CACX,MAAM,CAAE,IACT,CAEQ,2BAA4B,CAAC,iBAAI,CACxC,KAAK,CAAE,IAAI,CACX,MAAM,CAAE,IAAI,CACZ,UAAU,CAAE,IAAI,CAChB,SAAS,CAAE,IACZ,CAEA,qBAAQ,CACP,OAAO,CAAE,CACV"}'};const J=create_ssr_component((e,t,n,o)=>{let i=compute_rest_props(t,["src","fullscreen","fixed","transform","img_el","hidden","variant","max_height"]),{src:l=void 0}=t,{fullscreen:a=!1}=t,r,{fixed:s=!1}=t,{transform:d="translate(0px, 0px) scale(1)"}=t,{img_el:u=null}=t,{hidden:f=!1}=t,{variant:A$1="upload"}=t,{max_height:h=500}=t,_;createEventDispatcher();t.src===void 0&&n.src&&l!==void 0&&n.src(l),t.fullscreen===void 0&&n.fullscreen&&a!==void 0&&n.fullscreen(a),t.fixed===void 0&&n.fixed&&s!==void 0&&n.fixed(s),t.transform===void 0&&n.transform&&d!==void 0&&n.transform(d),t.img_el===void 0&&n.img_el&&u!==void 0&&n.img_el(u),t.hidden===void 0&&n.hidden&&f!==void 0&&n.hidden(f),t.variant===void 0&&n.variant&&A$1!==void 0&&n.variant(A$1),t.max_height===void 0&&n.max_height&&h!==void 0&&n.max_height(h),e.css.add(Dn);{r=l,_=l;const w=l;A(w).then(y=>{_===w&&(r=y);});}return ` <img${spread([{src:escape_attribute_value(r)},escape_object(i)],{classes:(s?"fixed":"")+" "+(f?"hidden":"")+" "+(A$1==="preview"?"preview":"")+" "+(A$1==="upload"?"slider":"")+" "+(a?"fullscreen":"")+" "+(a?"":"small")+" svelte-k63p1v",styles:{transform:d,"max-height":h&&!a?`${h}px`:null}})}${add_attribute("this",u,0)}>`});class Gn{container;image;scale;offsetX;offsetY;isDragging;lastX;lastY;initial_left_padding;initial_top_padding;initial_width;initial_height;subscribers;handleImageLoad;real_image_size={top:0,left:0,width:0,height:0};last_touch_distance;constructor(t,n){this.container=t,this.image=n,this.scale=1,this.offsetX=0,this.offsetY=0,this.isDragging=!1,this.lastX=0,this.lastY=0,this.initial_left_padding=0,this.initial_top_padding=0,this.initial_width=0,this.initial_height=0,this.subscribers=[],this.last_touch_distance=0,this.handleWheel=this.handleWheel.bind(this),this.handleMouseDown=this.handleMouseDown.bind(this),this.handleMouseMove=this.handleMouseMove.bind(this),this.handleMouseUp=this.handleMouseUp.bind(this),this.handleImageLoad=this.init.bind(this),this.handleTouchStart=this.handleTouchStart.bind(this),this.handleTouchMove=this.handleTouchMove.bind(this),this.handleTouchEnd=this.handleTouchEnd.bind(this),this.image.addEventListener("load",this.handleImageLoad),this.container.addEventListener("wheel",this.handleWheel),this.container.addEventListener("mousedown",this.handleMouseDown),document.addEventListener("mousemove",this.handleMouseMove),document.addEventListener("mouseup",this.handleMouseUp),this.container.addEventListener("touchstart",this.handleTouchStart),document.addEventListener("touchmove",this.handleTouchMove),document.addEventListener("touchend",this.handleTouchEnd),new ResizeObserver(i=>{for(const l of i)l.target===this.container&&(this.handleResize(),this.get_image_size(this.image));}).observe(this.container);}handleResize(){this.init();}init(){const t=this.container.getBoundingClientRect(),n=this.image.getBoundingClientRect();this.initial_left_padding=n.left-t.left,this.initial_top_padding=n.top-t.top,this.initial_width=n.width,this.initial_height=n.height,this.reset_zoom(),this.updateTransform();}reset_zoom(){this.scale=1,this.offsetX=0,this.offsetY=0,this.updateTransform();}handleMouseDown(t){const n=this.image.getBoundingClientRect();if(t.clientX>=n.left&&t.clientX<=n.right&&t.clientY>=n.top&&t.clientY<=n.bottom){if(t.preventDefault(),this.scale===1)return;this.isDragging=!0,this.lastX=t.clientX,this.lastY=t.clientY,this.image.style.cursor="grabbing";}}handleMouseMove(t){if(!this.isDragging)return;const n=t.clientX-this.lastX,o=t.clientY-this.lastY;this.offsetX+=n,this.offsetY+=o,this.lastX=t.clientX,this.lastY=t.clientY,this.updateTransform(),this.updateTransform();}handleMouseUp(){this.isDragging&&(this.constrain_to_bounds(!0),this.updateTransform(),this.isDragging=!1,this.image.style.cursor=this.scale>1?"grab":"zoom-in");}async handleWheel(t){t.preventDefault();const n=this.container.getBoundingClientRect(),o=this.image.getBoundingClientRect();if(t.clientX<o.left||t.clientX>o.right||t.clientY<o.top||t.clientY>o.bottom)return;const i=1.05,l=this.scale,a=-Math.sign(t.deltaY)>0?Math.min(15,l*i):Math.max(1,l/i);if(a===l)return;const r=t.clientX-n.left-this.initial_left_padding,s=t.clientY-n.top-this.initial_top_padding;this.scale=a,this.offsetX=this.compute_new_offset({cursor_position:r,current_offset:this.offsetX,new_scale:a,old_scale:l}),this.offsetY=this.compute_new_offset({cursor_position:s,current_offset:this.offsetY,new_scale:a,old_scale:l}),this.updateTransform(),this.constrain_to_bounds(),this.updateTransform(),this.image.style.cursor=this.scale>1?"grab":"zoom-in";}compute_new_position({position:t,scale:n,anchor_position:o}){return t-(t-o)*(n/this.scale)}compute_new_offset({cursor_position:t,current_offset:n,new_scale:o,old_scale:i}){return t-o/i*(t-n)}constrain_to_bounds(t=!1){if(this.scale===1){this.offsetX=0,this.offsetY=0;return}const n={top:this.real_image_size.top*this.scale+this.offsetY,left:this.real_image_size.left*this.scale+this.offsetX,width:this.real_image_size.width*this.scale,height:this.real_image_size.height*this.scale,bottom:this.real_image_size.top*this.scale+this.offsetY+this.real_image_size.height*this.scale,right:this.real_image_size.left*this.scale+this.offsetX+this.real_image_size.width*this.scale},o=this.real_image_size.left+this.real_image_size.width,i=this.real_image_size.top+this.real_image_size.height;t&&(n.top>this.real_image_size.top?this.offsetY=this.calculate_position(this.real_image_size.top,0,"y"):n.bottom<i&&(this.offsetY=this.calculate_position(i,1,"y")),n.left>this.real_image_size.left?this.offsetX=this.calculate_position(this.real_image_size.left,0,"x"):n.right<o&&(this.offsetX=this.calculate_position(o,1,"x")));}updateTransform(){this.notify({x:this.offsetX,y:this.offsetY,scale:this.scale});}destroy(){this.container.removeEventListener("wheel",this.handleWheel),this.container.removeEventListener("mousedown",this.handleMouseDown),document.removeEventListener("mousemove",this.handleMouseMove),document.removeEventListener("mouseup",this.handleMouseUp),this.container.removeEventListener("touchstart",this.handleTouchStart),document.removeEventListener("touchmove",this.handleTouchMove),document.removeEventListener("touchend",this.handleTouchEnd),this.image.removeEventListener("load",this.handleImageLoad);}subscribe(t){this.subscribers.push(t);}unsubscribe(t){this.subscribers=this.subscribers.filter(n=>n!==t);}notify({x:t,y:n,scale:o}){this.subscribers.forEach(i=>i({x:t,y:n,scale:o}));}handleTouchStart(t){t.preventDefault();const n=this.image.getBoundingClientRect(),o=t.touches[0];if(o.clientX>=n.left&&o.clientX<=n.right&&o.clientY>=n.top&&o.clientY<=n.bottom){if(t.touches.length===1&&this.scale>1)this.isDragging=!0,this.lastX=o.clientX,this.lastY=o.clientY;else if(t.touches.length===2){const i=t.touches[0],l=t.touches[1];this.last_touch_distance=Math.hypot(l.clientX-i.clientX,l.clientY-i.clientY);}}}get_image_size(t){if(!t)return;const n=t.parentElement?.getBoundingClientRect();if(!n)return;const o=t.naturalWidth/t.naturalHeight,i=n.width/n.height;let l,a;o>i?(l=n.width,a=n.width/o):(a=n.height,l=n.height*o);const r=(n.width-l)/2,s=(n.height-a)/2;this.real_image_size={top:s,left:r,width:l,height:a};}handleTouchMove(t){if(t.touches.length===1&&this.isDragging){t.preventDefault();const n=t.touches[0],o=n.clientX-this.lastX,i=n.clientY-this.lastY;this.offsetX+=o,this.offsetY+=i,this.lastX=n.clientX,this.lastY=n.clientY,this.updateTransform();}else if(t.touches.length===2){t.preventDefault();const n=t.touches[0],o=t.touches[1],i=Math.hypot(o.clientX-n.clientX,o.clientY-n.clientY);if(this.last_touch_distance===0){this.last_touch_distance=i;return}const l=i/this.last_touch_distance,a=this.scale,r=Math.min(15,Math.max(1,a*l));if(r===a){this.last_touch_distance=i;return}const s=this.container.getBoundingClientRect(),d=(n.clientX+o.clientX)/2-s.left-this.initial_left_padding,u=(n.clientY+o.clientY)/2-s.top-this.initial_top_padding;this.scale=r,this.offsetX=this.compute_new_offset({cursor_position:d,current_offset:this.offsetX,new_scale:r,old_scale:a}),this.offsetY=this.compute_new_offset({cursor_position:u,current_offset:this.offsetY,new_scale:r,old_scale:a}),this.updateTransform(),this.constrain_to_bounds(),this.updateTransform(),this.last_touch_distance=i,this.image.style.cursor=this.scale>1?"grab":"zoom-in";}}handleTouchEnd(t){this.isDragging&&(this.constrain_to_bounds(!0),this.updateTransform(),this.isDragging=!1),t.touches.length===0&&(this.last_touch_distance=0);}calculate_position(t,n,o){if(this.container.getBoundingClientRect(),o==="x"){const i=t,l=this.real_image_size.left+n*this.real_image_size.width;return i-l*this.scale}if(o==="y"){const i=t,l=this.real_image_size.top+n*this.real_image_size.height;return i-l*this.scale}return 0}}const Ln={code:".slider-wrap.svelte-eb87wk{user-select:none;height:100%;width:100%;position:relative;display:flex;align-items:center;justify-content:center}.limit_height.svelte-eb87wk img{max-height:500px}.image-container.svelte-eb87wk{height:100%;position:relative;min-width:var(--size-20)}",map:'{"version":3,"file":"SliderPreview.svelte","sources":["SliderPreview.svelte"],"sourcesContent":["<script lang=\\"ts\\">import Slider from \\"./Slider.svelte\\";\\nimport ImageEl from \\"./ImageEl.svelte\\";\\nimport { BlockLabel, Empty, IconButton, IconButtonWrapper, FullscreenButton } from \\"@gradio/atoms\\";\\nimport { Image, Download, Undo, Clear } from \\"@gradio/icons\\";\\nimport {} from \\"@gradio/client\\";\\nimport { DownloadLink } from \\"@gradio/wasm/svelte\\";\\nimport { ZoomableImage } from \\"./zoom\\";\\nimport { onMount } from \\"svelte\\";\\nimport { tweened } from \\"svelte/motion\\";\\nimport { createEventDispatcher } from \\"svelte\\";\\nexport let value = [null, null];\\nexport let label = void 0;\\nexport let show_download_button = true;\\nexport let show_label;\\nexport let i18n;\\nexport let position;\\nexport let layer_images = true;\\nexport let show_single = false;\\nexport let slider_color;\\nexport let show_fullscreen_button = true;\\nexport let fullscreen = false;\\nexport let el_width = 0;\\nexport let max_height;\\nexport let interactive = true;\\nconst dispatch = createEventDispatcher();\\nlet img;\\nlet slider_wrap;\\nlet image_container;\\nlet transform = tweened({ x: 0, y: 0, z: 1 }, {\\n    duration: 75\\n});\\nlet parent_el;\\n$: coords_at_viewport = get_coords_at_viewport(position, viewport_width, image_size.width, image_size.left, $transform.x, $transform.z);\\n$: style = layer_images ? `clip-path: inset(0 0 0 ${coords_at_viewport * 100}%)` : \\"\\";\\nfunction get_coords_at_viewport(viewport_percent_x, viewportWidth, image_width, img_offset_x, tx, scale) {\\n    const px_relative_to_image = viewport_percent_x * image_width;\\n    const pixel_position = px_relative_to_image + img_offset_x;\\n    const normalised_position = (pixel_position - tx) / scale;\\n    const percent_position = normalised_position / viewportWidth;\\n    return percent_position;\\n}\\nlet img_width = 0;\\nlet viewport_width = 0;\\nlet zoomable_image = null;\\nlet observer = null;\\nfunction init_image(img2, slider_wrap2) {\\n    if (!img2 || !slider_wrap2)\\n        return;\\n    zoomable_image?.destroy();\\n    observer?.disconnect();\\n    img_width = img2?.getBoundingClientRect().width || 0;\\n    viewport_width = slider_wrap2?.getBoundingClientRect().width || 0;\\n    zoomable_image = new ZoomableImage(slider_wrap2, img2);\\n    zoomable_image.subscribe(({ x, y, scale }) => {\\n        transform.set({ x, y, z: scale });\\n    });\\n    observer = new ResizeObserver((entries) => {\\n        for (const entry of entries) {\\n            if (entry.target === slider_wrap2) {\\n                viewport_width = entry.contentRect.width;\\n            }\\n            if (entry.target === img2) {\\n                img_width = entry.contentRect.width;\\n            }\\n        }\\n    });\\n    observer.observe(slider_wrap2);\\n    observer.observe(img2);\\n}\\n$: init_image(img, slider_wrap);\\nonMount(() => {\\n    return () => {\\n        zoomable_image?.destroy();\\n        observer?.disconnect();\\n    };\\n});\\nlet slider_wrap_parent;\\nlet image_size = { top: 0, left: 0, width: 0, height: 0 };\\nfunction handle_image_load(event) {\\n    image_size = event.detail;\\n}\\n<\/script>\\n\\n<BlockLabel {show_label} Icon={Image} label={label || i18n(\\"image.image\\")} />\\n{#if (value === null || value[0] === null || value[1] === null) && !show_single}\\n\\t<Empty unpadded_box={true} size=\\"large\\"><Image /></Empty>\\n{:else}\\n\\t<div class=\\"image-container\\" bind:this={image_container}>\\n\\t\\t<IconButtonWrapper>\\n\\t\\t\\t<IconButton\\n\\t\\t\\t\\tIcon={Undo}\\n\\t\\t\\t\\tlabel={i18n(\\"common.undo\\")}\\n\\t\\t\\t\\tdisabled={$transform.z === 1}\\n\\t\\t\\t\\ton:click={() => zoomable_image?.reset_zoom()}\\n\\t\\t\\t/>\\n\\t\\t\\t{#if show_fullscreen_button}\\n\\t\\t\\t\\t<FullscreenButton {fullscreen} on:fullscreen />\\n\\t\\t\\t{/if}\\n\\n\\t\\t\\t{#if show_download_button}\\n\\t\\t\\t\\t<DownloadLink\\n\\t\\t\\t\\t\\thref={value[1]?.url}\\n\\t\\t\\t\\t\\tdownload={value[1]?.orig_name || \\"image\\"}\\n\\t\\t\\t\\t>\\n\\t\\t\\t\\t\\t<IconButton Icon={Download} label={i18n(\\"common.download\\")} />\\n\\t\\t\\t\\t</DownloadLink>\\n\\t\\t\\t{/if}\\n\\t\\t\\t{#if interactive}\\n\\t\\t\\t\\t<IconButton\\n\\t\\t\\t\\t\\tIcon={Clear}\\n\\t\\t\\t\\t\\tlabel=\\"Remove Image\\"\\n\\t\\t\\t\\t\\ton:click={(event) => {\\n\\t\\t\\t\\t\\t\\tvalue = [null, null];\\n\\t\\t\\t\\t\\t\\tdispatch(\\"clear\\");\\n\\t\\t\\t\\t\\t\\tevent.stopPropagation();\\n\\t\\t\\t\\t\\t}}\\n\\t\\t\\t\\t/>\\n\\t\\t\\t{/if}\\n\\t\\t</IconButtonWrapper>\\n\\t\\t<div\\n\\t\\t\\tclass=\\"slider-wrap\\"\\n\\t\\t\\tbind:this={slider_wrap_parent}\\n\\t\\t\\tbind:clientWidth={el_width}\\n\\t\\t\\tclass:limit_height={!fullscreen}\\n\\t\\t>\\n\\t\\t\\t<Slider\\n\\t\\t\\t\\tbind:position\\n\\t\\t\\t\\t{slider_color}\\n\\t\\t\\t\\tbind:el={slider_wrap}\\n\\t\\t\\t\\tbind:parent_el\\n\\t\\t\\t\\t{image_size}\\n\\t\\t\\t>\\n\\t\\t\\t\\t<ImageEl\\n\\t\\t\\t\\t\\tsrc={value?.[0]?.url}\\n\\t\\t\\t\\t\\talt=\\"\\"\\n\\t\\t\\t\\t\\tloading=\\"lazy\\"\\n\\t\\t\\t\\t\\tbind:img_el={img}\\n\\t\\t\\t\\t\\tvariant=\\"preview\\"\\n\\t\\t\\t\\t\\ttransform=\\"translate({$transform.x}px, {$transform.y}px) scale({$transform.z})\\"\\n\\t\\t\\t\\t\\t{fullscreen}\\n\\t\\t\\t\\t\\t{max_height}\\n\\t\\t\\t\\t\\ton:load={handle_image_load}\\n\\t\\t\\t\\t/>\\n\\t\\t\\t\\t<ImageEl\\n\\t\\t\\t\\t\\tvariant=\\"preview\\"\\n\\t\\t\\t\\t\\tfixed={layer_images}\\n\\t\\t\\t\\t\\thidden={!value?.[1]?.url}\\n\\t\\t\\t\\t\\tsrc={value?.[1]?.url}\\n\\t\\t\\t\\t\\talt=\\"\\"\\n\\t\\t\\t\\t\\tloading=\\"lazy\\"\\n\\t\\t\\t\\t\\tstyle=\\"{style}; background: var(--block-background-fill);\\"\\n\\t\\t\\t\\t\\ttransform=\\"translate({$transform.x}px, {$transform.y}px) scale({$transform.z})\\"\\n\\t\\t\\t\\t\\t{fullscreen}\\n\\t\\t\\t\\t\\t{max_height}\\n\\t\\t\\t\\t\\ton:load={handle_image_load}\\n\\t\\t\\t\\t/>\\n\\t\\t\\t</Slider>\\n\\t\\t</div>\\n\\t</div>\\n{/if}\\n\\n<style>\\n\\t.slider-wrap {\\n\\t\\tuser-select: none;\\n\\t\\theight: 100%;\\n\\t\\twidth: 100%;\\n\\t\\tposition: relative;\\n\\t\\tdisplay: flex;\\n\\t\\talign-items: center;\\n\\t\\tjustify-content: center;\\n\\t}\\n\\n\\t.limit_height :global(img) {\\n\\t\\tmax-height: 500px;\\n\\t}\\n\\n\\t.image-container {\\n\\t\\theight: 100%;\\n\\t\\tposition: relative;\\n\\t\\tmin-width: var(--size-20);\\n\\t}</style>\\n"],"names":[],"mappings":"AAkKC,0BAAa,CACZ,WAAW,CAAE,IAAI,CACjB,MAAM,CAAE,IAAI,CACZ,KAAK,CAAE,IAAI,CACX,QAAQ,CAAE,QAAQ,CAClB,OAAO,CAAE,IAAI,CACb,WAAW,CAAE,MAAM,CACnB,eAAe,CAAE,MAClB,CAEA,2BAAa,CAAS,GAAK,CAC1B,UAAU,CAAE,KACb,CAEA,8BAAiB,CAChB,MAAM,CAAE,IAAI,CACZ,QAAQ,CAAE,QAAQ,CAClB,SAAS,CAAE,IAAI,SAAS,CACzB"}'};function Tn(e,t,n,o,i,l){return (e*n+o-i)/l/t}const Wn=create_ssr_component((e,t,n,o)=>{let i,l,a,r,{value:s=[null,null]}=t,{label:d=void 0}=t,{show_download_button:u=!0}=t,{show_label:f}=t,{i18n:A}=t,{position:h}=t,{layer_images:_=!0}=t,{show_single:b=!1}=t,{slider_color:w}=t,{show_fullscreen_button:y=!0}=t,{fullscreen:x=!1}=t,{el_width:z=0}=t,{max_height:c}=t,{interactive:m=!0}=t;createEventDispatcher();let v,g,E,S=tweened({x:0,y:0,z:1},{duration:75});r=subscribe(S,p=>a=p);let I,M=0,K=null,O=null;function B(p,W){!p||!W||(K?.destroy(),O?.disconnect(),p?.getBoundingClientRect().width,M=W?.getBoundingClientRect().width||0,K=new Gn(W,p),K.subscribe(({x:$,y:G,scale:zt})=>{S.set({x:$,y:G,z:zt});}),O=new ResizeObserver($=>{for(const G of $)G.target===W&&(M=G.contentRect.width),G.target===p&&G.contentRect.width;}),O.observe(W),O.observe(p));}let Y,R={top:0,left:0,width:0,height:0};t.value===void 0&&n.value&&s!==void 0&&n.value(s),t.label===void 0&&n.label&&d!==void 0&&n.label(d),t.show_download_button===void 0&&n.show_download_button&&u!==void 0&&n.show_download_button(u),t.show_label===void 0&&n.show_label&&f!==void 0&&n.show_label(f),t.i18n===void 0&&n.i18n&&A!==void 0&&n.i18n(A),t.position===void 0&&n.position&&h!==void 0&&n.position(h),t.layer_images===void 0&&n.layer_images&&_!==void 0&&n.layer_images(_),t.show_single===void 0&&n.show_single&&b!==void 0&&n.show_single(b),t.slider_color===void 0&&n.slider_color&&w!==void 0&&n.slider_color(w),t.show_fullscreen_button===void 0&&n.show_fullscreen_button&&y!==void 0&&n.show_fullscreen_button(y),t.fullscreen===void 0&&n.fullscreen&&x!==void 0&&n.fullscreen(x),t.el_width===void 0&&n.el_width&&z!==void 0&&n.el_width(z),t.max_height===void 0&&n.max_height&&c!==void 0&&n.max_height(c),t.interactive===void 0&&n.interactive&&m!==void 0&&n.interactive(m),e.css.add(Ln);let k,U,Q=e.head;do k=!0,e.head=Q,i=Tn(h,M,R.width,R.left,a.x,a.z),l=_?`clip-path: inset(0 0 0 ${i*100}%)`:"",B(v,g),U=`${validate_component(bt$1,"BlockLabel").$$render(e,{show_label:f,Icon:Xt,label:d||A("image.image")},{},{})} ${(s===null||s[0]===null||s[1]===null)&&!b?`${validate_component(kt,"Empty").$$render(e,{unpadded_box:!0,size:"large"},{},{default:()=>`${validate_component(Xt,"Image").$$render(e,{},{},{})}`})}`:`<div class="image-container svelte-eb87wk"${add_attribute("this",E,0)}>${validate_component(Ke$1,"IconButtonWrapper").$$render(e,{},{},{default:()=>`${validate_component(j,"IconButton").$$render(e,{Icon:pe$1,label:A("common.undo"),disabled:a.z===1},{},{})} ${y?`${validate_component(Le$1,"FullscreenButton").$$render(e,{fullscreen:x},{},{})}`:""} ${u?`${validate_component(C,"DownloadLink").$$render(e,{href:s[1]?.url,download:s[1]?.orig_name||"image"},{},{default:()=>`${validate_component(j,"IconButton").$$render(e,{Icon:Ut,label:A("common.download")},{},{})}`})}`:""} ${m?`${validate_component(j,"IconButton").$$render(e,{Icon:qt$1,label:"Remove Image"},{},{})}`:""}`})} <div class="${["slider-wrap svelte-eb87wk",x?"":"limit_height"].join(" ").trim()}"${add_attribute("this",Y,0)}>${validate_component(Bt,"Slider").$$render(e,{slider_color:w,image_size:R,position:h,el:g,parent_el:I},{position:p=>{h=p,k=!1;},el:p=>{g=p,k=!1;},parent_el:p=>{I=p,k=!1;}},{default:()=>`${validate_component(J,"ImageEl").$$render(e,{src:s?.[0]?.url,alt:"",loading:"lazy",variant:"preview",transform:"translate("+a.x+"px, "+a.y+"px) scale("+a.z+")",fullscreen:x,max_height:c,img_el:v},{img_el:p=>{v=p,k=!1;}},{})} ${validate_component(J,"ImageEl").$$render(e,{variant:"preview",fixed:_,hidden:!s?.[1]?.url,src:s?.[1]?.url,alt:"",loading:"lazy",style:l+"; background: var(--block-background-fill);",transform:"translate("+a.x+"px, "+a.y+"px) scale("+a.z+")",fullscreen:x,max_height:c},{},{})}`})}</div></div>`}`;while(!k);return r(),U}),Nn={code:"div.svelte-s6ybro{display:flex;position:absolute;top:var(--size-2);right:var(--size-2);justify-content:flex-end;gap:var(--spacing-sm);z-index:var(--layer-5)}",map:'{"version":3,"file":"ClearImage.svelte","sources":["ClearImage.svelte"],"sourcesContent":["<script lang=\\"ts\\">import { createEventDispatcher } from \\"svelte\\";\\nimport { IconButton } from \\"@gradio/atoms\\";\\nimport { Clear } from \\"@gradio/icons\\";\\nconst dispatch = createEventDispatcher();\\n<\/script>\\n\\n<div>\\n\\t<IconButton\\n\\t\\tIcon={Clear}\\n\\t\\tlabel=\\"Remove Image\\"\\n\\t\\ton:click={(event) => {\\n\\t\\t\\tdispatch(\\"remove_image\\");\\n\\t\\t\\tevent.stopPropagation();\\n\\t\\t}}\\n\\t/>\\n</div>\\n\\n<style>\\n\\tdiv {\\n\\t\\tdisplay: flex;\\n\\t\\tposition: absolute;\\n\\t\\ttop: var(--size-2);\\n\\t\\tright: var(--size-2);\\n\\t\\tjustify-content: flex-end;\\n\\t\\tgap: var(--spacing-sm);\\n\\t\\tz-index: var(--layer-5);\\n\\t}</style>\\n"],"names":[],"mappings":"AAkBC,iBAAI,CACH,OAAO,CAAE,IAAI,CACb,QAAQ,CAAE,QAAQ,CAClB,GAAG,CAAE,IAAI,QAAQ,CAAC,CAClB,KAAK,CAAE,IAAI,QAAQ,CAAC,CACpB,eAAe,CAAE,QAAQ,CACzB,GAAG,CAAE,IAAI,YAAY,CAAC,CACtB,OAAO,CAAE,IAAI,SAAS,CACvB"}'},Pn=create_ssr_component((e,t,n,o)=>(createEventDispatcher(),e.css.add(Nn),`<div class="svelte-s6ybro">${validate_component(j,"IconButton").$$render(e,{Icon:qt$1,label:"Remove Image"},{},{})} </div>`)),Vn={code:".upload-wrap.svelte-143b07a{display:flex;justify-content:center;align-items:center;height:100%;width:100%}.wrap.svelte-143b07a{width:100%}.half-wrap.svelte-143b07a{width:50%}.image-container.svelte-143b07a,.empty-wrap.svelte-143b07a{width:var(--size-full);height:var(--size-full)}.fixed.svelte-143b07a{--anim-block-background-fill:255, 255, 255;position:absolute;top:0;left:0;background-color:rgba(var(--anim-block-background-fill), 0.8);z-index:0}@media(prefers-color-scheme: dark){.fixed.svelte-143b07a{--anim-block-background-fill:31, 41, 55}}.side-by-side.svelte-143b07a img{width:50%;object-fit:contain}.empty-wrap.svelte-143b07a{pointer-events:none}.icon-buttons.svelte-143b07a{display:flex;position:absolute;right:8px;z-index:var(--layer-top);top:8px}",map:'{"version":3,"file":"Image.svelte","sources":["Image.svelte"],"sourcesContent":["<script lang=\\"ts\\">import Slider from \\"./Slider.svelte\\";\\nimport { createEventDispatcher, tick } from \\"svelte\\";\\nimport { BlockLabel, Empty, IconButton } from \\"@gradio/atoms\\";\\nimport { Download } from \\"@gradio/icons\\";\\nimport { Image } from \\"@gradio/icons\\";\\nimport {} from \\"@gradio/utils\\";\\nimport ClearImage from \\"./ClearImage.svelte\\";\\nimport ImageEl from \\"./ImageEl.svelte\\";\\nimport { Upload } from \\"@gradio/upload\\";\\nimport { DownloadLink } from \\"@gradio/wasm/svelte\\";\\nimport {} from \\"@gradio/client\\";\\nexport let value;\\nexport let label = void 0;\\nexport let show_label;\\nexport let root;\\nexport let position;\\nexport let upload_count = 2;\\nexport let show_download_button = true;\\nexport let slider_color;\\nexport let upload;\\nexport let stream_handler;\\nexport let max_file_size = null;\\nexport let i18n;\\nexport let max_height;\\nlet value_ = value || [null, null];\\nlet img;\\nlet el_width;\\nlet el_height;\\nasync function handle_upload({ detail }, n) {\\n    const new_value = [value[0], value[1]];\\n    if (detail.length > 1) {\\n        new_value[n] = detail[0];\\n    }\\n    else {\\n        new_value[n] = detail[n];\\n    }\\n    value = new_value;\\n    await tick();\\n    dispatch(\\"upload\\", new_value);\\n}\\nlet old_value = \\"\\";\\n$: if (JSON.stringify(value) !== old_value) {\\n    old_value = JSON.stringify(value);\\n    value_ = value;\\n}\\nconst dispatch = createEventDispatcher();\\nexport let dragging = false;\\n$: dispatch(\\"drag\\", dragging);\\n<\/script>\\n\\n<BlockLabel {show_label} Icon={Image} label={label || i18n(\\"image.image\\")} />\\n\\n<div\\n\\tdata-testid=\\"image\\"\\n\\tclass=\\"image-container\\"\\n\\tbind:clientWidth={el_width}\\n\\tbind:clientHeight={el_height}\\n>\\n\\t{#if value?.[0]?.url || value?.[1]?.url}\\n\\t\\t<ClearImage\\n\\t\\t\\ton:remove_image={() => {\\n\\t\\t\\t\\tposition = 0.5;\\n\\t\\t\\t\\tvalue = [null, null];\\n\\t\\t\\t\\tdispatch(\\"clear\\");\\n\\t\\t\\t}}\\n\\t\\t/>\\n\\t{/if}\\n\\t{#if value?.[1]?.url}\\n\\t\\t<div class=\\"icon-buttons\\">\\n\\t\\t\\t{#if show_download_button}\\n\\t\\t\\t\\t<DownloadLink\\n\\t\\t\\t\\t\\thref={value[1].url}\\n\\t\\t\\t\\t\\tdownload={value[1].orig_name || \\"image\\"}\\n\\t\\t\\t\\t>\\n\\t\\t\\t\\t\\t<IconButton Icon={Download} />\\n\\t\\t\\t\\t</DownloadLink>\\n\\t\\t\\t{/if}\\n\\t\\t</div>\\n\\t{/if}\\n\\t<Slider\\n\\t\\tbind:position\\n\\t\\tdisabled={upload_count == 2 || !value?.[0]}\\n\\t\\t{slider_color}\\n\\t>\\n\\t\\t<div\\n\\t\\t\\tclass=\\"upload-wrap\\"\\n\\t\\t\\tstyle:display={upload_count === 2 ? \\"flex\\" : \\"block\\"}\\n\\t\\t\\tclass:side-by-side={upload_count === 2}\\n\\t\\t>\\n\\t\\t\\t{#if !value_?.[0]}\\n\\t\\t\\t\\t<div class=\\"wrap\\" class:half-wrap={upload_count === 1}>\\n\\t\\t\\t\\t\\t<Upload\\n\\t\\t\\t\\t\\t\\tbind:dragging\\n\\t\\t\\t\\t\\t\\tfiletype=\\"image/*\\"\\n\\t\\t\\t\\t\\t\\ton:load={(e) => handle_upload(e, 0)}\\n\\t\\t\\t\\t\\t\\tdisable_click={!!value?.[0]}\\n\\t\\t\\t\\t\\t\\t{root}\\n\\t\\t\\t\\t\\t\\tfile_count=\\"multiple\\"\\n\\t\\t\\t\\t\\t\\t{upload}\\n\\t\\t\\t\\t\\t\\t{stream_handler}\\n\\t\\t\\t\\t\\t\\t{max_file_size}\\n\\t\\t\\t\\t\\t>\\n\\t\\t\\t\\t\\t\\t<slot />\\n\\t\\t\\t\\t\\t</Upload>\\n\\t\\t\\t\\t</div>\\n\\t\\t\\t{:else}\\n\\t\\t\\t\\t<ImageEl\\n\\t\\t\\t\\t\\tvariant=\\"upload\\"\\n\\t\\t\\t\\t\\tsrc={value_[0]?.url}\\n\\t\\t\\t\\t\\talt=\\"\\"\\n\\t\\t\\t\\t\\tbind:img_el={img}\\n\\t\\t\\t\\t\\t{max_height}\\n\\t\\t\\t\\t/>\\n\\t\\t\\t{/if}\\n\\n\\t\\t\\t{#if !value_?.[1] && upload_count === 2}\\n\\t\\t\\t\\t<Upload\\n\\t\\t\\t\\t\\tbind:dragging\\n\\t\\t\\t\\t\\tfiletype=\\"image/*\\"\\n\\t\\t\\t\\t\\ton:load={(e) => handle_upload(e, 1)}\\n\\t\\t\\t\\t\\tdisable_click={!!value?.[1]}\\n\\t\\t\\t\\t\\t{root}\\n\\t\\t\\t\\t\\tfile_count=\\"multiple\\"\\n\\t\\t\\t\\t\\t{upload}\\n\\t\\t\\t\\t\\t{stream_handler}\\n\\t\\t\\t\\t\\t{max_file_size}\\n\\t\\t\\t\\t>\\n\\t\\t\\t\\t\\t<slot />\\n\\t\\t\\t\\t</Upload>\\n\\t\\t\\t{:else if !value_?.[1] && upload_count === 1}\\n\\t\\t\\t\\t<div\\n\\t\\t\\t\\t\\tclass=\\"empty-wrap fixed\\"\\n\\t\\t\\t\\t\\tstyle:width=\\"{el_width * (1 - position)}px\\"\\n\\t\\t\\t\\t\\tstyle:transform=\\"translateX({el_width * position}px)\\"\\n\\t\\t\\t\\t\\tclass:white-icon={!value?.[0]?.url}\\n\\t\\t\\t\\t>\\n\\t\\t\\t\\t\\t<Empty unpadded_box={true} size=\\"large\\"><Image /></Empty>\\n\\t\\t\\t\\t</div>\\n\\t\\t\\t{:else if value_?.[1]}\\n\\t\\t\\t\\t<ImageEl\\n\\t\\t\\t\\t\\tvariant=\\"upload\\"\\n\\t\\t\\t\\t\\tsrc={value_[1].url}\\n\\t\\t\\t\\t\\talt=\\"\\"\\n\\t\\t\\t\\t\\tfixed={upload_count === 1}\\n\\t\\t\\t\\t\\ttransform=\\"translate(0px, 0px) scale(1)\\"\\n\\t\\t\\t\\t\\t{max_height}\\n\\t\\t\\t\\t/>\\n\\t\\t\\t{/if}\\n\\t\\t</div>\\n\\t</Slider>\\n</div>\\n\\n<style>\\n\\t.upload-wrap {\\n\\t\\tdisplay: flex;\\n\\t\\tjustify-content: center;\\n\\t\\talign-items: center;\\n\\t\\theight: 100%;\\n\\t\\twidth: 100%;\\n\\t}\\n\\n\\t.wrap {\\n\\t\\twidth: 100%;\\n\\t}\\n\\n\\t.half-wrap {\\n\\t\\twidth: 50%;\\n\\t}\\n\\t.image-container,\\n\\t.empty-wrap {\\n\\t\\twidth: var(--size-full);\\n\\t\\theight: var(--size-full);\\n\\t}\\n\\n\\t.fixed {\\n\\t\\t--anim-block-background-fill: 255, 255, 255;\\n\\t\\tposition: absolute;\\n\\t\\ttop: 0;\\n\\t\\tleft: 0;\\n\\t\\tbackground-color: rgba(var(--anim-block-background-fill), 0.8);\\n\\t\\tz-index: 0;\\n\\t}\\n\\n\\t@media (prefers-color-scheme: dark) {\\n\\t\\t.fixed {\\n\\t\\t\\t--anim-block-background-fill: 31, 41, 55;\\n\\t\\t}\\n\\t}\\n\\n\\t.side-by-side :global(img) {\\n\\t\\t/* width: 100%; */\\n\\t\\twidth: 50%;\\n\\t\\tobject-fit: contain;\\n\\t}\\n\\n\\t.empty-wrap {\\n\\t\\tpointer-events: none;\\n\\t}\\n\\n\\t.icon-buttons {\\n\\t\\tdisplay: flex;\\n\\t\\tposition: absolute;\\n\\t\\tright: 8px;\\n\\t\\tz-index: var(--layer-top);\\n\\t\\ttop: 8px;\\n\\t}</style>\\n"],"names":[],"mappings":"AAyJC,2BAAa,CACZ,OAAO,CAAE,IAAI,CACb,eAAe,CAAE,MAAM,CACvB,WAAW,CAAE,MAAM,CACnB,MAAM,CAAE,IAAI,CACZ,KAAK,CAAE,IACR,CAEA,oBAAM,CACL,KAAK,CAAE,IACR,CAEA,yBAAW,CACV,KAAK,CAAE,GACR,CACA,+BAAgB,CAChB,0BAAY,CACX,KAAK,CAAE,IAAI,WAAW,CAAC,CACvB,MAAM,CAAE,IAAI,WAAW,CACxB,CAEA,qBAAO,CACN,4BAA4B,CAAE,aAAa,CAC3C,QAAQ,CAAE,QAAQ,CAClB,GAAG,CAAE,CAAC,CACN,IAAI,CAAE,CAAC,CACP,gBAAgB,CAAE,KAAK,IAAI,4BAA4B,CAAC,CAAC,CAAC,GAAG,CAAC,CAC9D,OAAO,CAAE,CACV,CAEA,MAAO,uBAAuB,IAAI,CAAE,CACnC,qBAAO,CACN,4BAA4B,CAAE,UAC/B,CACD,CAEA,4BAAa,CAAS,GAAK,CAE1B,KAAK,CAAE,GAAG,CACV,UAAU,CAAE,OACb,CAEA,0BAAY,CACX,cAAc,CAAE,IACjB,CAEA,4BAAc,CACb,OAAO,CAAE,IAAI,CACb,QAAQ,CAAE,QAAQ,CAClB,KAAK,CAAE,GAAG,CACV,OAAO,CAAE,IAAI,WAAW,CAAC,CACzB,GAAG,CAAE,GACN"}'},Fn=create_ssr_component((e,t,n,o)=>{let{value:i}=t,{label:l=void 0}=t,{show_label:a}=t,{root:r}=t,{position:s}=t,{upload_count:d=2}=t,{show_download_button:u=!0}=t,{slider_color:f}=t,{upload:A}=t,{stream_handler:h}=t,{max_file_size:_=null}=t,{i18n:b}=t,{max_height:w}=t,y=i||[null,null],x,z,c="";const m=createEventDispatcher();let{dragging:v=!1}=t;t.value===void 0&&n.value&&i!==void 0&&n.value(i),t.label===void 0&&n.label&&l!==void 0&&n.label(l),t.show_label===void 0&&n.show_label&&a!==void 0&&n.show_label(a),t.root===void 0&&n.root&&r!==void 0&&n.root(r),t.position===void 0&&n.position&&s!==void 0&&n.position(s),t.upload_count===void 0&&n.upload_count&&d!==void 0&&n.upload_count(d),t.show_download_button===void 0&&n.show_download_button&&u!==void 0&&n.show_download_button(u),t.slider_color===void 0&&n.slider_color&&f!==void 0&&n.slider_color(f),t.upload===void 0&&n.upload&&A!==void 0&&n.upload(A),t.stream_handler===void 0&&n.stream_handler&&h!==void 0&&n.stream_handler(h),t.max_file_size===void 0&&n.max_file_size&&_!==void 0&&n.max_file_size(_),t.i18n===void 0&&n.i18n&&b!==void 0&&n.i18n(b),t.max_height===void 0&&n.max_height&&w!==void 0&&n.max_height(w),t.dragging===void 0&&n.dragging&&v!==void 0&&n.dragging(v),e.css.add(Vn);let g,E,S=e.head;do g=!0,e.head=S,JSON.stringify(i)!==c&&(c=JSON.stringify(i),y=i),m("drag",v),E=`${validate_component(bt$1,"BlockLabel").$$render(e,{show_label:a,Icon:Xt,label:l||b("image.image")},{},{})} <div data-testid="image" class="image-container svelte-143b07a">${i?.[0]?.url||i?.[1]?.url?`${validate_component(Pn,"ClearImage").$$render(e,{},{},{})}`:""} ${i?.[1]?.url?`<div class="icon-buttons svelte-143b07a">${u?`${validate_component(C,"DownloadLink").$$render(e,{href:i[1].url,download:i[1].orig_name||"image"},{},{default:()=>`${validate_component(j,"IconButton").$$render(e,{Icon:Ut},{},{})}`})}`:""}</div>`:""} ${validate_component(Bt,"Slider").$$render(e,{disabled:d==2||!i?.[0],slider_color:f,position:s},{position:I=>{s=I,g=!1;}},{default:()=>`<div class="${["upload-wrap svelte-143b07a",d===2?"side-by-side":""].join(" ").trim()}"${add_styles({display:d===2?"flex":"block"})}>${y?.[0]?`${validate_component(J,"ImageEl").$$render(e,{variant:"upload",src:y[0]?.url,alt:"",max_height:w,img_el:x},{img_el:I=>{x=I,g=!1;}},{})}`:`<div class="${["wrap svelte-143b07a",d===1?"half-wrap":""].join(" ").trim()}">${validate_component(ge$1,"Upload").$$render(e,{filetype:"image/*",disable_click:!!i?.[0],root:r,file_count:"multiple",upload:A,stream_handler:h,max_file_size:_,dragging:v},{dragging:I=>{v=I,g=!1;}},{default:()=>`${o.default?o.default({}):""}`})}</div>`} ${!y?.[1]&&d===2?`${validate_component(ge$1,"Upload").$$render(e,{filetype:"image/*",disable_click:!!i?.[1],root:r,file_count:"multiple",upload:A,stream_handler:h,max_file_size:_,dragging:v},{dragging:I=>{v=I,g=!1;}},{default:()=>`${o.default?o.default({}):""}`})}`:`${!y?.[1]&&d===1?`<div class="${["empty-wrap fixed svelte-143b07a",i?.[0]?.url?"":"white-icon"].join(" ").trim()}"${add_styles({width:`${z*(1-s)}px`,transform:`translateX(${z*s}px)`})}>${validate_component(kt,"Empty").$$render(e,{unpadded_box:!0,size:"large"},{},{default:()=>`${validate_component(Xt,"Image").$$render(e,{},{},{})}`})}</div>`:`${y?.[1]?`${validate_component(J,"ImageEl").$$render(e,{variant:"upload",src:y[1].url,alt:"",fixed:d===1,transform:"translate(0px, 0px) scale(1)",max_height:w},{},{})}`:""}`}`}</div>`})} </div>`;while(!g);return E}),Zn=create_ssr_component((e,t,n,o)=>{let{value:i=[null,null]}=t,{upload:l}=t,{stream_handler:a}=t,{label:r}=t,{show_label:s}=t,{i18n:d}=t,{root:u}=t,{upload_count:f=1}=t,{dragging:A}=t,{max_height:h}=t,{max_file_size:_=null}=t;t.value===void 0&&n.value&&i!==void 0&&n.value(i),t.upload===void 0&&n.upload&&l!==void 0&&n.upload(l),t.stream_handler===void 0&&n.stream_handler&&a!==void 0&&n.stream_handler(a),t.label===void 0&&n.label&&r!==void 0&&n.label(r),t.show_label===void 0&&n.show_label&&s!==void 0&&n.show_label(s),t.i18n===void 0&&n.i18n&&d!==void 0&&n.i18n(d),t.root===void 0&&n.root&&u!==void 0&&n.root(u),t.upload_count===void 0&&n.upload_count&&f!==void 0&&n.upload_count(f),t.dragging===void 0&&n.dragging&&A!==void 0&&n.dragging(A),t.max_height===void 0&&n.max_height&&h!==void 0&&n.max_height(h),t.max_file_size===void 0&&n.max_file_size&&_!==void 0&&n.max_file_size(_);let b,w,y=e.head;do b=!0,e.head=y,w=`  ${validate_component(Fn,"Image").$$render(e,{slider_color:"var(--border-color-primary)",position:.5,root:u,label:r,show_label:s,upload_count:f,stream_handler:a,upload:l,max_file_size:_,max_height:h,i18n:d,value:i,dragging:A},{value:x=>{i=x,b=!1;},dragging:x=>{A=x,b=!1;}},{default:()=>`${o.default?o.default({}):""}`})}`;while(!b);return w});let Hn=!1;const li=create_ssr_component((e,t,n,o)=>{let i,{value_is_output:l=!1}=t,{elem_id:a=""}=t,{elem_classes:r=[]}=t,{visible:s=!0}=t,{value:d=[null,null]}=t,u=[null,null],{label:f}=t,{show_label:A}=t,{show_download_button:h}=t,{root:_}=t,{height:b}=t,{width:w}=t,{container:y=!0}=t,{scale:x=null}=t,{min_width:z=void 0}=t,{loading_status:c}=t,{interactive:m}=t,{placeholder:v=void 0}=t,{show_fullscreen_button:g}=t,E=!1,{input_ready:S}=t,{slider_position:I}=t,{upload_count:M=1}=t,{slider_color:K="var(--border-color-primary)"}=t,{max_height:O}=t,{gradio:B}=t;let Y,R;t.value_is_output===void 0&&n.value_is_output&&l!==void 0&&n.value_is_output(l),t.elem_id===void 0&&n.elem_id&&a!==void 0&&n.elem_id(a),t.elem_classes===void 0&&n.elem_classes&&r!==void 0&&n.elem_classes(r),t.visible===void 0&&n.visible&&s!==void 0&&n.visible(s),t.value===void 0&&n.value&&d!==void 0&&n.value(d),t.label===void 0&&n.label&&f!==void 0&&n.label(f),t.show_label===void 0&&n.show_label&&A!==void 0&&n.show_label(A),t.show_download_button===void 0&&n.show_download_button&&h!==void 0&&n.show_download_button(h),t.root===void 0&&n.root&&_!==void 0&&n.root(_),t.height===void 0&&n.height&&b!==void 0&&n.height(b),t.width===void 0&&n.width&&w!==void 0&&n.width(w),t.container===void 0&&n.container&&y!==void 0&&n.container(y),t.scale===void 0&&n.scale&&x!==void 0&&n.scale(x),t.min_width===void 0&&n.min_width&&z!==void 0&&n.min_width(z),t.loading_status===void 0&&n.loading_status&&c!==void 0&&n.loading_status(c),t.interactive===void 0&&n.interactive&&m!==void 0&&n.interactive(m),t.placeholder===void 0&&n.placeholder&&v!==void 0&&n.placeholder(v),t.show_fullscreen_button===void 0&&n.show_fullscreen_button&&g!==void 0&&n.show_fullscreen_button(g),t.input_ready===void 0&&n.input_ready&&S!==void 0&&n.input_ready(S),t.slider_position===void 0&&n.slider_position&&I!==void 0&&n.slider_position(I),t.upload_count===void 0&&n.upload_count&&M!==void 0&&n.upload_count(M),t.slider_color===void 0&&n.slider_color&&K!==void 0&&n.slider_color(K),t.max_height===void 0&&n.max_height&&O!==void 0&&n.max_height(O),t.gradio===void 0&&n.gradio&&B!==void 0&&n.gradio(B);let k,U,Q=e.head;do k=!0,e.head=Q,i=Math.max(0,Math.min(100,I))/100,S=!Hn,JSON.stringify(d)!==JSON.stringify(u)&&(u=d,B.dispatch("change"),l||B.dispatch("input")),U=`  ${!m||d?.[1]&&d?.[0]?`${validate_component(mt$1,"Block").$$render(e,{visible:s,variant:"solid",border_mode:Y?"focus":"base",padding:!1,elem_id:a,elem_classes:r,height:b||void 0,width:w,allow_overflow:!1,container:y,scale:x,min_width:z,fullscreen:E},{fullscreen:p=>{E=p,k=!1;}},{default:()=>`${validate_component(zA,"StatusTracker").$$render(e,Object.assign({},{autoscroll:B.autoscroll},{i18n:B.i18n},c),{},{})} ${validate_component(Wn,"StaticImage").$$render(e,{fullscreen:E,interactive:m,label:f,show_label:A,show_download_button:h,i18n:B.i18n,show_fullscreen_button:g,position:i,slider_color:K,max_height:O,value:d},{value:p=>{d=p,k=!1;}},{})}`})}`:`${validate_component(mt$1,"Block").$$render(e,{visible:s,variant:d===null?"dashed":"solid",border_mode:Y?"focus":"base",padding:!1,elem_id:a,elem_classes:r,height:b||void 0,width:w,allow_overflow:!1,container:y,scale:x,min_width:z},{},{default:()=>`${validate_component(zA,"StatusTracker").$$render(e,Object.assign({},{autoscroll:B.autoscroll},{i18n:B.i18n},c),{},{})} ${validate_component(Zn,"ImageUploader").$$render(e,{root:_,label:f,show_label:A,upload_count:M,max_file_size:B.max_file_size,i18n:B.i18n,upload:(...p)=>B.client.upload(...p),stream_handler:B.client?.stream,max_height:O,this:R,value:d,dragging:Y},{this:p=>{R=p,k=!1;},value:p=>{d=p,k=!1;},dragging:p=>{Y=p,k=!1;}},{default:()=>`${`${validate_component(_e$1,"UploadText").$$render(e,{i18n:B.i18n,type:"image",placeholder:v},{},{})}`}`})}`})}`}`;while(!k);return U});

export { li as default };
//# sourceMappingURL=Index63-DtfEYzqo.js.map
